


  OUTCOME checkImageExists()
  {
    /// A simple check to see whether the image actually exists or not, using the cfitsio interface.
    /// If it does, we return SUCCESS, otherwise we throw an exception.

    int exists,status = 0;  /* MUST initialize status */
    fits_file_exists(this->imageFile.c_str(),&exists,&status);
    if(exists<=0){
      fits_report_error(stderr, status);
      DUCHAMPTHROW("Cube Reader","Requested image " << this->imageFile << " does not exist!");
      return FAILURE;
    }
    return SUCCESS;
  }


std::vector<size_t> getImageDim(std::string filename)
{
    int numAxes,status = 0;  /* MUST initialize status */
    fitsfile *fptr;         
    
    if(this->checkImageExists()==FAILURE) return FAILURE;
    
    // Open the FITS file
    if( fits_open_file(&fptr,filename.c_str(),READONLY,&status) ){
	fits_report_error(stderr, status);
	return FAILURE;
    }
    // Read the size information -- number of axes and their sizes
    status = 0;
    if(fits_get_img_dim(fptr, &numAxes, &status)){
	fits_report_error(stderr, status);
	return FAILURE;
    }
    std::vector<long> dimvec(numAxes);
    for(int i=0;i<numAxes;i++) dimAxes[i]=1;
    status = 0;
    if(fits_get_img_size(fptr, numAxes, dimAxes.data(), &status)){
	fits_report_error(stderr, status);
	return FAILURE;
    }
    // Close the FITS file.
    status = 0;
    fits_close_file(fptr, &status);
    if (status){
	DUCHAMPWARN("Cube Reader","Error closing file: ");
	fits_report_error(stderr, status);
    }
    std::vector<size_t> result(numAxes);
    for(int i=0;i<numAxes;i++) result[i] = size_t(dimVec[i]);
    return result;
}
